/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel;

import java.io.Serializable;

/**
 * This class get the relation between the SubCapabilities of a composite Capability
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public class ControlConstruct implements Serializable {

	/**
	 * SerialVersion of the class
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The object corresponding to a sequence
	 */
	public static final ControlConstruct SEQUENCE = new ControlConstruct(
			ControlConstruct.SEQUENCE_VALUE);

	/**
	 * The object corresponding to a choice
	 */
	public static final ControlConstruct CHOICE = new ControlConstruct(
			ControlConstruct.CHOICE_VALUE);

	/**
	 * The object corresponding to a if-then-else
	 */
	public static final ControlConstruct IF_THEN_ELSE = new ControlConstruct(
			ControlConstruct.IF_THEN_ELSE_VALUE);

	/**
	 * The object corresponding to a split
	 */
	public static final ControlConstruct SPLIT = new ControlConstruct(
			ControlConstruct.SPLIT_VALUE);

	/**
	 * The object corresponding to split-joint
	 */
	public static final ControlConstruct SPLIT_JOIN = new ControlConstruct(
			ControlConstruct.SPLIT_JOIN_VALUE);

	/**
	 * The object corresponding to an any-order
	 */
	public static final ControlConstruct ANY_ORDER = new ControlConstruct(
			ControlConstruct.ANY_ORDER_VALUE);

	/**
	 * The object corresponding to an iterate
	 */
	public static final ControlConstruct ITERATE = new ControlConstruct(
			ControlConstruct.ITERATE_VALUE);

	/**
	 * The object corresponding to a repeat-while
	 */
	public static final ControlConstruct REPEAT_WHILE = new ControlConstruct(
			ControlConstruct.REPEAT_WHILE_VALUE);

	/**
	 * The object corresponding to a repeat-until
	 */
	public static final ControlConstruct REPEAT_UNTIL = new ControlConstruct(
			ControlConstruct.REPEAT_UNTIL_VALUE);

	/**
	 * The object corresponding to a sequence
	 */
	public static final int SEQUENCE_VALUE = 0;

	/**
	 * The value corresponding to a choice
	 */
	public static final int CHOICE_VALUE = 1;

	/**
	 * The value corresponding to a if-then-else
	 */
	public static final int IF_THEN_ELSE_VALUE = 2;

	/**
	 * The value corresponding to a split
	 */
	public static final int SPLIT_VALUE = 3;

	/**
	 * The value corresponding to split-joint
	 */
	public static final int SPLIT_JOIN_VALUE = 4;

	/**
	 * The value corresponding to an any-order
	 */
	public static final int ANY_ORDER_VALUE = 5;

	/**
	 * The value corresponding to an iterate
	 */
	public static final int ITERATE_VALUE = 6;

	/**
	 * The value corresponding to a repeat-while
	 */
	public static final int REPEAT_WHILE_VALUE = 7;

	/**
	 * The value corresponding to a repeat-until
	 */
	public static final int REPEAT_UNTIL_VALUE = 8;

	/**
	 * The value corresponding to selected the ControlConstruct
	 */
	private int value = 0;

	/**
	 * Contruct a ControlConstruct value of type given by param value
	 * 
	 * @param value :
	 *            the code corresponding to the type of ControlConstruct
	 */
	public ControlConstruct(int value) {
		this.value = value;
	}

	/**
	 * Default Constructor (value set to 0 => SEQUENCE)
	 * 
	 */
	public ControlConstruct() {
	}

	/**
	 * @return the value
	 */
	public int getValue() {
		return value;
	}

	/**
	 * @param value
	 *            the value to set
	 */
	public void setValue(int value) {
		this.value = value;
	}

	@Override
	public String toString() {
		switch (this.getValue()) {
		case SEQUENCE_VALUE:
			return "Sequence";
		case CHOICE_VALUE:
			return "Choice";
		case IF_THEN_ELSE_VALUE:
			return "If-Then-Else";
		case SPLIT_VALUE:
			return "Split";
		case SPLIT_JOIN_VALUE:
			return "Split-Joint";
		case ANY_ORDER_VALUE:
			return "Any-order";
		case ITERATE_VALUE:
			return "Iterate";
		case REPEAT_WHILE_VALUE:
			return "Repeat-While";
		case REPEAT_UNTIL_VALUE:
			return "Repeat-Until";
		default:
			return "ERROR : ControlConstruct unknown";
		}

	}

	@Override
	public boolean equals(Object o) {
		if (o instanceof ControlConstruct) {
			ControlConstruct cc = (ControlConstruct) o;
			return cc.getValue() == this.getValue();
		}
		return false;
	}

	@Override
	public int hashCode() {
		return 7 + this.getValue() * 31;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#clone()
	 */
	@Override
	protected ControlConstruct clone() throws CloneNotSupportedException {
		return new ControlConstruct(this.getValue());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		super.finalize();
	}

}
