/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel;

/**
 * This class describe an Input of a Capability.
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public class Input extends IO implements Transferable<Input> {

	/**
	 * SerialVersion of the class
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Constructor of the class, the value of "ontology" and "localName" are
	 * initialized with empty string.
	 * 
	 * @param name
	 *            the name of the concept (whithout ontology name)
	 */
	public Input(String name) {
		super(name, IO.UNKNOWN, IO.UNKNOWN);
	}

	/**
	 * Constructor of an Input Object by giving the local URI of the individual
	 * and the rdfType of this one.
	 * 
	 * @param localNameURI
	 *            the URI of the individual coding the Input
	 * @param rdfTypeURI
	 *            the rdf Type (= semantic Type of the Input)
	 */
	public Input(String localNameURI, String rdfTypeURI) {
		super(localNameURI, rdfTypeURI);
	}

	/**
	 * Constructor of the class
	 * 
	 * @param name
	 *            the name of the concept (whithout ontology name)
	 * @param ontology
	 *            the name of the ontology from which the concept is extract
	 * @param localName
	 *            the name of the Input in the file
	 */
	public Input(String name, String ontology, String localName) {
		super(name, ontology, localName);
	}

	@Override
	public String toString() {
		return "Input" + (isProvided() ? "(P)" : "") + "["
				+ this.getLocalName() + "=" + this.getSemanticType() + "(" + this.getGroundingValue() + ")]";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.IO#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object _in) {
		if (_in instanceof Input) {
			Input in = (Input) _in;
			return super.equals(in);
		} else {
			return false;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.IO#hashCode()
	 */
	@Override
	public int hashCode() {
		return super.hashCode();
	}

	public int containsMoreInformation(Input otherObject) {
		return otherObject.countNullOrUnknowField()
				- this.countNullOrUnknowField();
	}

	public void transfertInformation(Input otherObject) {
		super.transfertInformation(otherObject);
	}

}
