/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel;

import java.util.List;
import java.util.Map;

import fr.inria.icocoa.servicemodel.toolsobjects.ProcessElements;

/**
 * 
 * This interface provide a list of function needed to permit the generation of
 * the BPEL script
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Feb 2, 2007 1:12:54 PM
 */
public interface ProcessInterface {

	/**
	 * Get the service name
	 */
	public abstract String getServiceName();

	/**
	 * Get the list of required capability
	 */
	public abstract List<String> getRequiredTask();

	/**
	 * Get the conversation associated to a capability
	 * 
	 * @param capability_uri
	 *            the URI of the capability we want to get the conversation
	 *            name, , null if the capability didn't exist
	 */
	public abstract String getTaskProcess(String capability_uri);

	/**
	 * Test if a Capability has an atomic process
	 * 
	 * @param capability_uri
	 *            the URI of the capability we want to get the conversation
	 *            name.
	 * @return true if the conversation is an OWL-S AtomicProcess, false
	 *         otherwise
	 */
	public abstract boolean isAtomic(String capability_uri);

	/**
	 * Test if a Capability has an composite process
	 * 
	 * @param capability_uri
	 *            the URI of the capability we want to get the conversation
	 *            name.
	 * @return true if the conversation is an OWL-S CompositProcess, false
	 *         otherwise
	 */
	public abstract boolean isComposite(String capability_uri);

	/**
	 * Get the input parameters of a process
	 * 
	 * @param capability_uri
	 *            the URI of the capability we want to get the conversation
	 *            name.
	 * @return the Map of Inputs associated to the given capability, null if the
	 *         capability didn't exist
	 */
	public abstract Map<String, Input> getInputs(String capability_uri);

	/**
	 * Get the output parameters of a process
	 * 
	 * @param capability_uri
	 *            the URI of the capability we want to get the conversation
	 *            name.
	 * @return the map of Outputs associated to the given capability, null if
	 *         the capability didn't exist
	 */
	public abstract Map<String, Output> getOutputs(String capability_uri);

	/**
	 * Get the Process control construct + elements + bindings
	 * 
	 * @param capability_uri
	 *            the URI of the capability we want to get the conversation
	 *            name.
	 * @return a composite object containg all information about the process,
	 *         null if the capability didn't exist
	 */
	public abstract ProcessElements getProcessElements(String capability_uri);

	/**
	 * Get the capability associated with the given name
	 * 
	 * @param capabilityName 
	 *            the name of the required capability
	 * @return a Capability object, null if co Capability is associated with
	 *         this name
	 */
	public abstract Capability getRequiredCapability(String capabilityName);
}