/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel;

import java.io.Serializable;

/**
 * 
 * THis class will contains the Qos Property (an XML description of all QoS constraints) associated with a service.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jun 7, 2007 3:18:51 PM
 */
public class QoSProperty implements Serializable {

	/**
	 * SerialVersion of the class
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Part of text in a Service description corresponding to the QoS.
	 */
	private String qoSXMLPart = "";

	/**
	 * @param qoSXMLPart
	 */
	public QoSProperty(String qoSXMLPart) {
		this.qoSXMLPart = qoSXMLPart;
	}

	/**
	 * @return the qoSXMLPart
	 */
	public String getQoSXMLPart() {
		return qoSXMLPart;
	}

	/**
	 * @param qoSXMLPart
	 *            the qoSXMLPart to set
	 */
	public void setQoSXMLPart(String qoSXMLPart) {
		this.qoSXMLPart = qoSXMLPart;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result
				+ ((qoSXMLPart == null) ? 0 : qoSXMLPart.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final QoSProperty other = (QoSProperty) obj;
		if (qoSXMLPart == null) {
			if (other.qoSXMLPart != null)
				return false;
		} else if (!qoSXMLPart.equals(other.qoSXMLPart))
			return false;
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return this.getQoSXMLPart();
	}

}
