/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel;

/**
 * This interface provided some operation usefull to detect if two objects that must be similar have this property. It permit to ensure this property if it is not the case.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Jul 27, 2007 5:26:01 PM
 */
public interface Transferable<T> {

	/**
	 * Compare the number of variable filled
	 * 
	 * @param otherObject
	 *            the object to compare with
	 * @return a positive number if other information fill more information that
	 *         the current object; a negative number if it is the contrary, and
	 *         zero if they fill exactly the same number of field
	 */
	int containsMoreInformation(T otherObject);

	/**
	 * Transfert all the field from otherObject to the current object (erase the
	 * already existing content)
	 * 
	 * @param otherObject
	 *            the object from which value will be extracted.
	 */
	void transfertInformation(T otherObject);

	/**
	 * Count the number of Null or UNKNOW Field of the current value
	 * 
	 * @return this number (this is a positive or null value)
	 */
	int countNullOrUnknowField();
}
