/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel.automaton;

import java.util.Iterator;

import fr.inria.icocoa.servicemodel.Capability;

/**
 * This class implements some static method to create automaton depending the control construct use by composite capabilities.
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public class AutomataFactory {

	/**
	 * Create an automaton for Sequence
	 * @param c the composite capability that required an automaton
	 * @return the automaton corresponding to the given composite capability
	 */
	public static Automaton generateSequenceAutomaton(Capability c) {
		Integer nbrStates = Integer.valueOf(0);
		Automaton tempAutomaton = new Automaton();
		tempAutomaton.setInitialState(new InitialState(c.getLocalName()
				+ (nbrStates++).toString()));
		Iterator it = c.getSubCapabilities().iterator();
		State previousState = tempAutomaton.getInitialState();
		tempAutomaton.addState(previousState);
		while (it.hasNext()) {
			Capability subCapability = (Capability) it.next();
			Symbol newSymbol;
			if (!tempAutomaton.containsASymbolNamed(subCapability.getLocalName())) {
				newSymbol = new Symbol(subCapability.getLocalName());
				newSymbol.setServiceName(c.getService().getLocalName());
				tempAutomaton.getSymbols().add(newSymbol);
			} else {
				newSymbol = tempAutomaton.getSymbolNamed(subCapability.getLocalName());
			}
			State newState = new State(c.getLocalName()
					+ (nbrStates++).toString());
			tempAutomaton.addTransition(previousState, newSymbol, newState);
			tempAutomaton.addState(newState);
			if (!tempAutomaton.getSymbols().contains(c.getLocalName()))
				tempAutomaton.addSymbol(c.getLocalName());
			previousState = newState;
		}
		previousState.setFinal(true);;
		tempAutomaton.addFinalState(previousState);
		return (tempAutomaton);
	}

	/**
	 * Create an automaton for Choice
	 * @param c the composite capability that required an automaton
	 * @return the automaton corresponding to the given composite capability
	 */
	public static Automaton generateChoiceAutomaton(Capability c) {
		Integer nbrStates = Integer.valueOf(0);
		Automaton tempAutomaton = new Automaton();
		tempAutomaton.setInitialState(new InitialState(c.getLocalName()
				+ (nbrStates++).toString()));
		tempAutomaton.addState(tempAutomaton.getInitialState());
		Iterator it = c.getSubCapabilities().iterator();
		while (it.hasNext()) {
			Capability childCapability = (Capability) it.next();
			Symbol newSymbol;
			if (!tempAutomaton.containsASymbolNamed(childCapability.getLocalName())) {
				newSymbol = new Symbol(childCapability.getLocalName());
				newSymbol.setServiceName(c.getService().getLocalName());
				tempAutomaton.getSymbols().add(newSymbol);
			} else {
				newSymbol = tempAutomaton.getSymbolNamed(childCapability.getLocalName());
			}
			FinalState childState = new FinalState(c.getLocalName()
					+ (nbrStates++).toString());
			tempAutomaton.addTransition(tempAutomaton.getInitialState(),
				newSymbol, childState);
			tempAutomaton.addState(childState);
			tempAutomaton.addFinalState(childState);
			if (!tempAutomaton.getSymbols().contains(
				childCapability.getLocalName()))
				tempAutomaton.addSymbol(childCapability.getLocalName());
		}
		return (tempAutomaton);

	}

	/**
	 * Create an automaton for AnyOrder
	 * @param c the composite capability that required an automaton
	 * @return the automaton corresponding to the given composite capability
	 */
	public static Automaton generateAnyOrderAutomaton(Capability c) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Create an automaton for IfThenElse
	 * @param c the composite capability that required an automaton
	 * @return the automaton corresponding to the given composite capability
	 */
	public static Automaton generateIfThenElseAutomaton(Capability c) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Create an automaton for Iterate
	 * @param c the composite capability that required an automaton
	 * @return the automaton corresponding to the given composite capability
	 */
	public static Automaton generateIterateAutomaton(Capability c) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Create an automaton for RepeatUntil
	 * @param c the composite capability that required an automaton
	 * @return the automaton corresponding to the given composite capability
	 */
	public static Automaton generateRepeatUntilAutomaton(Capability c) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Create an automaton for RepeatWhile
	 * @param c the composite capability that required an automaton
	 * @return the automaton corresponding to the given composite capability
	 */
	public static Automaton generateRepeatWhileAutomaton(Capability c) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Create an automaton for Split
	 * @param c the composite capability that required an automaton
	 * @return the automaton corresponding to the given composite capability
	 */
	public static Automaton generateSplitAutomaton(Capability c) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Create an automaton for SplitJoin
	 * @param c the composite capability that required an automaton
	 * @return the automaton corresponding to the given composite capability
	 */
	public static Automaton generateSplitJoinAutomaton(Capability c) {
		// TODO Auto-generated method stub
		return null;
	}

}
