/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel.automaton;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.servicemodel.Service;

/**
 * This class provided some method to aggregate automaton
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public class AutomataUnion extends Automaton {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * This union is used to link together the automata that correspond the
	 * provided capabilities of a service, epsilon-transitions are generated to
	 * link the start states of each provided capability with a new start state
	 * @param listOfAutomata the list of automaton to aggregate
	 * @return the resulting automaton
	 */
	@SuppressWarnings("unchecked")
	public static Automaton makeUnion(List listOfAutomata) {
		if (listOfAutomata.size() > 1) {
			Automaton resultingAutomaton = new Automaton();
			resultingAutomaton.setInitialState(new InitialState("Toto"));/*
			 * to
			 * be
			 * changed
			 */
			resultingAutomaton.getStates().add(
					resultingAutomaton.getInitialState());
			List transitionsToAdd = new ArrayList();
			List transitionTablesToAdd = new ArrayList();
			for (Iterator listOfAutomataIt = listOfAutomata.iterator(); listOfAutomataIt
					.hasNext();) {
				Automaton currentAutomaton = (Automaton) listOfAutomataIt
						.next();
				resultingAutomaton.addFinalStatesList(currentAutomaton
						.getFinalStates());
				// resultingAutomaton.addStatesList(currentAutomaton.getStates());
				// resultingAutomaton.addSymbols(currentAutomaton.getSymbols());
				transitionTablesToAdd
						.add(currentAutomaton.getTransitionTable());
				transitionsToAdd.add(new Transition(resultingAutomaton
						.getInitialState(), Automaton.EPSILON, currentAutomaton
						.getInitialState()));
				/*
				 * for(Iterator
				 * finalStatesIt=currentAutomaton.getFinalStates().iterator();finalStatesIt.hasNext();){
				 * State currentFinalState = (State)finalStatesIt.next();
				 * transitionsToAdd.add(new Transition(currentFinalState,
				 * automaton.EPSILON, resultingAutomaton.getInitialState())); }
				 */
			}

			resultingAutomaton.addTransitionTablesList(transitionTablesToAdd);
			resultingAutomaton.addTransitionList(transitionsToAdd);
			return (resultingAutomaton);
		} else
			return ((Automaton) listOfAutomata.get(0));
	}

	/**
	 * This method collect the automaton to aggregate from the list of service 
	 * @param listOfServices the list of service which automaton must be aggregated
	 * @return the resulting automaton of the aggregation (cf {@link #makeUnion(List)})
	 */
	@SuppressWarnings("unchecked")
	public static Automaton connectServices(List listOfServices) {
		List automataList = new ArrayList();
		List transitionsToAdd = new ArrayList();
		for (Iterator it = listOfServices.iterator(); it.hasNext();) {
			Service s = (Service) it.next();
			Automaton currentAutoToConnect = s.getAutomaton();
			DebugPrinter.print("service automaton " + s.getLocalName() + " =\n"
					+ currentAutoToConnect, DebugPrinter.INFO);
			automataList.add(currentAutoToConnect);
		}
		Automaton resultingAutomaton = makeUnion(automataList);
		for (Iterator it = resultingAutomaton.getFinalStates().iterator(); it
				.hasNext();) {
			State currentFinalState = (State) it.next();
			transitionsToAdd.add(new Transition(currentFinalState,
					Automaton.EPSILON, resultingAutomaton.getInitialState()));
		}
		resultingAutomaton.addTransitionList(transitionsToAdd);
		return (resultingAutomaton);
	}

}
