/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel.automaton;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

/**
 * This class gather some staic mathod to manipulate double {@link HashMap} (HashMap that contains HashMap).<br><br>
 * The basic structure of the double HashMap is an Hashmap, where each object of the HashMap is another Hashmap. The objects stored into the second HashMap could be a List (then used method getValueAtList(...) and putValueAtList(...) to manipulate such object. 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public class DoubleHashMap implements Serializable {
	
	/**
	 * the version of the class for serialization
	 */
	private static final long serialVersionUID = 1L;

	
	// --------------------Methodes de manipulation des hashmap deux
	// dimensions----------------------------------------------------------------------------------------------------
	
	/**
	 * Add a new object in the "DoubleHashMap"
	 * @param premiereMap the double HashMap
	 * @param value the object to insert
	 * @param clef1 the key of the object to insert in the first HashMap
	 * @param clef2 the key of the object to insert in the second HashMap 
	 */
	@SuppressWarnings("unchecked")
	public static void putValue(HashMap premiereMap, Object value, Object clef1,
			Object clef2) {

		if (value == null || clef1 == null || clef2 == null) {
			// System.out.println("Null "+((State)clef1).getName()+"
			// "+((Symbol)clef2).getName());
			return;
		}

		Map secondMap = (Map) premiereMap.get(clef1);
		if (secondMap == null) {
			secondMap = new HashMap();

		}
		secondMap.put(clef2, value);
		premiereMap.put(clef1, secondMap);
		// System.out.println("g insr "+((State)clef1).getName()+"
		// "+((Symbol)clef2).getName());
	}

	// ------------------------------------------------------------------------------------------
	/**
	 * Get the object contains in the double hashmap foloowing the two keys
	 * @param premiereMap the DoubleHashMap to search in
	 * @param clef1 the key of the required object in the first HashMap
	 * @param clef2 the key of the required ibject in the second HashMap
	 * @return the required object, <b>null</b> if one on the key does not exist.	
	 */
	public static Object getValue(HashMap premiereMap, Object clef1, Object clef2) {
		if (premiereMap == null) {
			System.out.println("map 1==null");
			return null;
		}
		Map secondMap = (Map) premiereMap.get(clef1);
		if (secondMap == null) {
			return null;
		}
		return secondMap.get(clef2);
	}

	// ------------------------------------------------------------------------------------------
	/**
	 * Put a new object in a list. This list correspond to the object required in the given HashMap associated with clef1 then clef2
	 * @param premiereMap the DoubleHashMap in which we want to add a value
	 * @param value the object to add
	 * @param clef1 the first key of the list we want to add a value
	 * @param clef2 the second key of the list we want to add a value
	 */
	@SuppressWarnings("unchecked")
	public static void putValueAtList(HashMap premiereMap, Object value, Object clef1,
			Object clef2) {
		List v = (List) getValue(premiereMap, clef1, clef2);
		if (v == null) {
			v = new Vector();
		}
		v.add(value);
		putValue(premiereMap, v, clef1, clef2);
	}

	// ------------------------------------------------------------------------------------------
	/**
	 * Get the value [index] the list stored as an object in the double HashMap, and located according to the first and secon key
	 * @param premiereMap the DoubleHashMap
	 * @param clef1 the first key that permit to locate the selected list
	 * @param clef2 the second key that permit to locate the selected list
	 * @param index the index of the required element in the selected list
	 * @return the required element of the selected list
	 */
	public static Object getValueAtList(HashMap premiereMap, Object clef1,
			Object clef2, int index) {
		List v = (List) getValue(premiereMap, clef1, clef2);
		return (v.get(index));
	}
	// --------------------------------------------------------------------

}
