/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel.automaton;

import java.io.Serializable;

/**
 * This class represents the different state of the Automaton
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public class State implements Serializable {
	/**
	 * the serial version for serialization
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The name of the state
	 */
	protected String name;
	
	/**
	 * indicate if the state is the Intial State of the automaton
	 */
	protected boolean isInitial;
	
	/**
	 * Indicate if the state is a final state of the automaton 
	 */
	protected boolean isFinal;

	/**
	 * Default constructor of the class
	 *
	 */
	public State() {
	}

	/**
	 * Create a new state from basic informationd
	 * @param name the name of the state
	 * @param isInitial indicate if the state in an initial state or not
	 * @param isFinal indicate if the state is final or not
	 */
	public State(String name, boolean isInitial, boolean isFinal) {
		this.name = name;
		this.isInitial = isInitial;
		this.isFinal = isFinal;
	}

/**
 * Create a state (neither initial , neither final) from its name
 * @param name the name of the state.
 */
	public State(String name) {
		this.name = name;
		// this.IsInitial=false;
		// this.IsFinal=false;

	}

	// ------------------------------------------------------------------------------
	public void setName(String name) {
		this.name = name;
	}

	// ------------------------------------------------------------------------------
	public String getName() {
		return (this.name);
	}

	// ------------------------------------------------------------------------------
	public boolean isInitialState() {
		return (this.isInitial);
	}

	// ------------------------------------------------------------------------------
	public boolean isFinalState() {
		return (this.isFinal);
	}
	
	

	// ------------------------------------------------------------------------------

	/**
	 * @param isFinal the isFinal to set
	 */
	public void setFinal(boolean isFinal) {
		this.isFinal = isFinal;
	}

	/**
	 * @param isInitial the isInitial to set
	 */
	public void setInitial(boolean isInitial) {
		this.isInitial = isInitial;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		this.destroy();
		super.finalize();
	}

	/**
	 * Destroy the object (delete all links with other object)
	 * 
	 */
	private void destroy() {
		this.name = null;
	}
}
