/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel.automaton;

import java.io.Serializable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import fr.inria.icocoa.servicemodel.automaton.qos.QoSAttribute;

/**
 * This class represent a Symbol. A symbol indicate the action to do when following a transition between two states.
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public class Symbol implements Serializable {
	/**
	 * serial version of the class 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The name of the symbol
	 */
	private String name;
	
	/**
	 * The service associated to this symbol
	 */
	private String serviceName;
	/*
	 * private List inputs; private List outputs; private List QoSRequirements;
	 */
	/**
	 * The list of qos attribute associated to the symbol
	 */
	private List qoSAttributes = new LinkedList();

	/**
	 * Create a symbol from its name
	 * @param name the name of the Symbol
	 */
	public Symbol(String name) {
		this.name = name;
	}

	// ----------------------------------------------------------------
	public String getName() {
		return (this.name);
	}

	@Override
	public boolean equals(Object s) {
		if (s instanceof Symbol)
		{
			Symbol _s = (Symbol)s  ;
			return this.getName().equals(_s.getName());
		}else return false;
	
	}


	/**
	 * Get a QOSAttribut from its name
	 * @param attributeName the name of the required attribute
	 * @return the QosAttribute object, <b>null</b> if no attribute with such name exists.
	 */
	public QoSAttribute getAttributeNamed(int attributeName) {
		for (Iterator it = qoSAttributes.iterator(); it.hasNext();) {
			QoSAttribute attribute = (QoSAttribute) it.next();
			if (attribute.getAttributeType() == attributeName) {
				return (attribute);
			}
		}
		return null;
	}

	public void setServiceName(String ServiceName) {
		this.serviceName = ServiceName;
	}

	/**
	 * @return the serviceName
	 */
	public String getServiceName() {
		return serviceName;
	}

	@Override
	public String toString() {
		return super.toString() + " = [" + this.getName() + ","
				+ this.getServiceName() + "]";
	}

	/**
	 * Get the name of the service without the reference to the corresponding ontology
	 * @return the short name of the symbol (without reference to ontology)
	 */
	public String getNameSplited() {
		// we cut the ontology name and the Sharp to get only the Name
		return this.name.substring(this.name.lastIndexOf("#") + 1);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		this.destroy();
		super.finalize();
	}

	/**
	 * Destroy the object (delete all links with other object)
	 * 
	 */
	private void destroy() {
		this.name = null;
		this.serviceName = null;
		this.qoSAttributes.clear();
		this.qoSAttributes = null;

	}

}
