/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel.automaton;

import java.io.Serializable;
import java.util.HashMap;

/**
 * This object describe the transition between two state. It gather the stating state of the transitio, the ending state, and the symbol followed.
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public class Transition implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The state that start the transition
	 */
	private State StartState;
	
	/**
	 * The state that end the transition
	 */
	private State EndState;
	
	/**
	 * The symbol followed during the transition
	 */
	private Symbol Symbol;
	
	/**
	 * A map use to store the QoS attribute associated to the transition
	 */
	private HashMap QoSAttributes = new HashMap(2);
	
	/**
	 * An index use during calculation of the global QoS 
	 */
	private int indexInTheFormulaTree;

	/**
	 * Create a new Transition object from its basic properties
	 * @param StartState the starting state of the transition
	 * @param Symbol the followed symbol
	 * @param EndState the ending state of the transition
	 */
	public Transition(State StartState, Symbol Symbol, State EndState) {
		this.StartState = StartState;
		this.Symbol = Symbol;
		this.EndState = EndState;
		this.QoSAttributes = new HashMap();
	}

	

	/**
	 * @return the endState
	 */
	public State getEndState() {
		return EndState;
	}



	/**
	 * @param endState the endState to set
	 */
	public void setEndState(State endState) {
		EndState = endState;
	}



	/**
	 * @return the qoSAttributes
	 */
	public HashMap getQoSAttributes() {
		return QoSAttributes;
	}



	/**
	 * @param qoSAttributes the qoSAttributes to set
	 */
	public void setQoSAttributes(HashMap qoSAttributes) {
		QoSAttributes = qoSAttributes;
	}



	/**
	 * @return the startState
	 */
	public State getStartState() {
		return StartState;
	}



	/**
	 * @param startState the startState to set
	 */
	public void setStartState(State startState) {
		StartState = startState;
	}



	/**
	 * @return the symbol
	 */
	public Symbol getSymbol() {
		return Symbol;
	}



	/**
	 * @param symbol the symbol to set
	 */
	public void setSymbol(Symbol symbol) {
		Symbol = symbol;
	}



	/**
	 * @return the indexInTheFormulaTree
	 */
	public int getIndexInTheFormulaTree() {
		return indexInTheFormulaTree;
	}



	/**
	 * @param indexInTheFormulaTree the indexInTheFormulaTree to set
	 */
	public void setIndexInTheFormulaTree(int indexInTheFormulaTree) {
		this.indexInTheFormulaTree = indexInTheFormulaTree;
	}



	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		this.destroy();
		super.finalize();
	}

	/**
	 * Destroy the object (delete all links with other object)
	 * 
	 */
	private void destroy() {
		this.EndState = null;
		this.QoSAttributes.clear();
		this.QoSAttributes = null;
		this.StartState = null;
		this.Symbol = null;

	}

}
