/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel.automaton.qos;

import java.util.Iterator;
import java.util.List;

/**
 * This class represent a QoSAttrtibute dedicated to represent Latency
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public class LatencyAttribute extends QoSAttribute {
	
	/**
	 * Create an new object from the initial value
	 * @param value the initial value of the LatencyAttribute
	 */
	public LatencyAttribute(double value) {
		this.setAttributeType(QoSAttribute.LATENCY);
		this.setBestValue(0);
		this.setValue(value);
		this.setIncreases(true);
	}

	public double sequenceLogic(List listOfParameters) {
		double sum = 0;
		for (Iterator it = listOfParameters.iterator(); it.hasNext();) {
			Double parameter = (Double) it.next();
			// System.out.println("l_seq_parameter "+ parameter.doubleValue());
			sum += parameter.doubleValue();
		}
		return (sum);
	}

	public double choiceLogic(List listOfParameters) {
		double max = 0;
		for (Iterator it = listOfParameters.iterator(); it.hasNext();) {
			Double parameter = (Double) it.next();
			// System.out.println("l_ch_parameter "+ parameter.doubleValue());
			if (parameter.doubleValue() > max)
				max = parameter.doubleValue();
		}
		return (max);
	}

	@Override
	public int getAttributeType() {
		return QoSAttribute.LATENCY;
	}
	/*
	 * public boolean isAcceptable(QoSAttributeInterface anotherValue){
	 * if(anotherValue.getValue() <= this.getValue()){ return(true); }else{
	 * return(false); } }
	 */

}
