/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel.automaton.qos;

/**
 * This class implements an attribute of QoS. This attributes are used during composition to remove composition that doesn't respect the required QoS.
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public abstract class QoSAttribute implements QoSAttributeInterface {

	/**
	 * The type of attribut (Latency/Availability)
	 */
	private int AttributeType;
	
	/**
	 * The best value of this Attribute
	 */
	private int BestValue;
	
	/**
	 * The current value of this attribute
	 */
	private double Value;
	
	/**
	 * Indicate if this attribut increase or not
	 */
	private boolean Increases;
	
	/**
	 * Static value for type Latency of QoS Attribute
	 */
	public static final int LATENCY = 0;
	
	/**
	 * Static value for type Availability of QoS Attribute
	 */
	public static final int AVAILABILITY = 1;
	
	/**
	 * Static value for type Latency of QoS Attribute (object version)
	 */
	public static final Integer LATENCY_OBJ = Integer.valueOf(QoSAttribute.LATENCY);
	
	/**
	 * Static value for type Availability of QoS Attribute (object version)
	 */
	public static final Integer AVAILABILITY_OBJ = Integer.valueOf(QoSAttribute.AVAILABILITY);

	public QoSAttribute() {
	}

	public void setAttributeType(int AttributeType) {
		this.AttributeType = AttributeType;
	}

	public int getAttributeType() {
		return AttributeType;
	}

	public void setBestValue(int BestValue) {
		this.BestValue = BestValue;
	}

	public int getBestValue() {
		return BestValue;
	}

	public void setValue(double Value) {
		this.Value = Value;
	}

	public double getValue() {
		if (this.Value == -1)
			return (this.getBestValue());
		else
			return Value;
	}

	public void setIncreases(boolean Increases) {
		this.Increases = Increases;
	}

	public boolean isIncreases() {
		return Increases;
	}
}
