/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel.automaton.qos;

import java.util.List;

/**
 * This interface define the minimum set of operations that need to be implemented by QoSAttribute.
 * 
 * <br>Thoses operation will be call during algorithm to obtain the global QoS value for each attribute.
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public interface QoSAttributeInterface {
	
	/**
	 * Get the global value of QoS for a sequence
	 * @param listOfParameters the list of QoSAttribute that have to be consider so as to calculate the global value of QoS
	 * @return the global QoS value for the QoSAttribute
	 */
	public double sequenceLogic(List listOfParameters);

	/**
	 * Get the global value of QoS for a choice
	 * @param listOfParameters the list of QoSAttribute that have to be consider so as to calculate the global value of QoS
	 * @return the global QoS value for the QoSAttribute
	 */
	public double choiceLogic(List listOfParameters);

	/**
	 * Get the current value of QoSAttribute 
	 * @return the value of the QoSAttribute
	 */
	public double getValue();

	/**
	 * Get the current value of QoSAttribute 
	 * @param value the new value of the QoSAttribute
	 */
	public void setValue(double value);

	/**
	 * Get the type of QoSAttribute
	 * @return the value of atttribute type
	 */
	public int getAttributeType();
	// public boolean isAcceptable(QoSAttributeInterface anotherValue);

}
