/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel.automaton.qos;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.servicemodel.automaton.Automaton;
import fr.inria.icocoa.servicemodel.automaton.State;
import fr.inria.icocoa.servicemodel.automaton.Symbol;
import fr.inria.icocoa.servicemodel.automaton.Transition;

/**
 * This class represent the tree of QoSAttribute generated during composition of services. This tree will be used to compute the global QoSAttribute so as to remove composition that can't respect the QoS requirement.
 * 
 * @author <a href="mailto:sonia.ben_mokhtar@inria.fr>@inria.fr">Sonia Ben
 *         Mokhtar</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project -
 *         INRIA</a>
 */
public class QoSFormulaTree {
	
	/**
	 * The associated automaton
	 */
	private Automaton automaton;

	/**
	 * The map of QoSParameters
	 */
	private HashMap QoSParameterMap;

	/**
	 * The root of the tree
	 */
	private TreeNode rootNode;

	
	public List list = new ArrayList();

	int leafIndex = 0;

	int actualIndex = 0;

	int QoSAttributeActualIndex = 0;

	private int NumberOfLeafs;

	public QoSFormulaTree(Automaton automaton, HashMap QoSParameterMap) {
		this.automaton = automaton;
		this.QoSParameterMap = QoSParameterMap;
		this.rootNode = new TreeNode();
		this.rootNode.setIsRoot(true);
		this.rootNode.setLabel("Root");

		construct(automaton.getInitialState(), this.rootNode);
		this.setNumberOfLeafs(leafIndex);
		// inordre(this.rootNode);
	}

		private void construct(State state, TreeNode node) {
		// considere que l'automate de la tache ne boucle pas !! sinon
		// stackoverflow
		List followingSymbolList = automaton.getFollowingSymbols(state);
		if (followingSymbolList == null) {
			node.add(NodesFactory.createDefaultNode(this.QoSParameterMap));
			leafIndex++;
		} else {
			if (followingSymbolList.size() == 1) {
				Symbol followingSymbol = (Symbol) followingSymbolList.get(0); // regler
				// le
				// pb
				// de
				// la
				// transition....
				TreeNode child = NodesFactory.createSequenceNode(this.QoSParameterMap);
				child.add(NodesFactory.createDefaultNode(this.QoSParameterMap));
				node.add(child);
				State newState = automaton.getFollowingState(state,
					followingSymbol).get(0);
				String transitionKey = Automaton.getTransition(state.getName(),
					followingSymbol.getName(), newState.getName());
				Transition transition = automaton.getTransitionProperties().get(
					transitionKey);
				transition.setIndexInTheFormulaTree(leafIndex);
				// System.out.println(transitionKey + " " + leafIndex);
				leafIndex++;
				construct(newState, child);
			} else {// choice case
				TreeNode child = NodesFactory.createChoiceNode(this.QoSParameterMap);
				node.add(child);
				for (Iterator it = followingSymbolList.iterator(); it.hasNext();) {
					Symbol followingSymbol = (Symbol) it.next();
					TreeNode petitChild = NodesFactory.createSequenceNode(this.QoSParameterMap);
					petitChild.add(NodesFactory.createDefaultNode(this.QoSParameterMap));
					child.add(petitChild);
					State newState = automaton.getFollowingState(state,
						followingSymbol).get(0);
					String transitionKey = Automaton.getTransition(
						state.getName(), followingSymbol.getName(),
						newState.getName());
					Transition transition = automaton.getTransitionProperties().get(
						transitionKey);
					transition.setIndexInTheFormulaTree(leafIndex);
					// System.out.println(transitionKey + " " + leafIndex);
					leafIndex++;
					// attention on considere que l'automate de la tache est
					// deterministe...pb...
					construct(newState, petitChild);
				}
			}
		}
	}

	
	@SuppressWarnings("unchecked")
	public HashMap evaluateAll(TreeNode node) {
		if (node.getLabel().equals("Root")) {
			return evaluateAll((TreeNode) node.getChildren().get(0));
		} else {
			if (node.isLeaf()) {
				HashMap interfacesList = node.getQoSAttributesMap();
				HashMap resu = new HashMap();
				for (Iterator it = interfacesList.keySet().iterator(); it.hasNext();) {
					QoSAttributeInterface inter = (QoSAttributeInterface) interfacesList.get(it.next());
					if (inter instanceof LatencyAttribute) {
						resu.put(QoSAttribute.LATENCY_OBJ, new Double(
								inter.getValue()));
					} else {
						if (inter instanceof AvailabilityAttribute) {
							resu.put(QoSAttribute.AVAILABILITY_OBJ, new Double(
									inter.getValue()));
						}
					}
				}
				return (resu);
			} else {
				if (node.getLabel().equals("SEQUENCE")) {
					List parameters = new ArrayList();
					for (Iterator it = node.getChildren().iterator(); it.hasNext();) {
						TreeNode child = (TreeNode) it.next();
						parameters.add(evaluateAll(child));
					}
					HashMap resu = new HashMap();
					for (Iterator it2 = this.QoSParameterMap.keySet().iterator(); it2.hasNext();) {
						Integer actualQoSAttribute = (Integer) it2.next();
						List actual = new ArrayList();
						for (Iterator it = parameters.iterator(); it.hasNext();) {
							actual.add(((HashMap) it.next()).get(actualQoSAttribute));
						}

						resu.put(
							actualQoSAttribute,
							new Double(
									((QoSAttributeInterface) (node.getQoSAttributesMap().get(actualQoSAttribute))).sequenceLogic(actual)));
					}
					return (resu);
				} else {
					List parameters = new ArrayList();
					for (Iterator it = node.getChildren().iterator(); it.hasNext();) {
						TreeNode child = (TreeNode) it.next();
						parameters.add(evaluateAll(child));
					}
					HashMap resu = new HashMap();
					for (Iterator it2 = this.QoSParameterMap.keySet().iterator(); it2.hasNext();) {
						Integer actualQoSAttribute = ((Integer) it2.next());
						List actual = new ArrayList();
						for (Iterator it = parameters.iterator(); it.hasNext();) {
							actual.add(((HashMap) it.next()).get(actualQoSAttribute));
						}
						resu.put(
							actualQoSAttribute,
							new Double(
									((QoSAttributeInterface) (node.getQoSAttributesMap().get(actualQoSAttribute))).choiceLogic(actual)));
					}
					return (resu);
				}
			}
		}
	}

	@SuppressWarnings("unchecked")
	public double evaluate(TreeNode node, Integer QoSParameter) {
		if (node.getLabel().equals("Root")) {
			return evaluate((TreeNode) node.getChildren().get(0), QoSParameter);
		} else {
			if (node.isLeaf()) {
				return (((QoSAttributeInterface) node.getQoSAttributesMap().get(
					QoSParameter)).getValue());
			} else {
				if (node.getLabel().equals("SEQUENCE")) {
					List parameters = new ArrayList();
					for (Iterator it = node.getChildren().iterator(); it.hasNext();) {
						TreeNode child = (TreeNode) it.next();
						parameters.add(new Double(evaluate(child, QoSParameter)));
					}
					return (((QoSAttributeInterface) node.getQoSAttributesMap().get(
						QoSParameter)).sequenceLogic(parameters));
				} else {
					List parameters = new ArrayList();
					for (Iterator it = node.getChildren().iterator(); it.hasNext();) {
						TreeNode child = (TreeNode) it.next();
						parameters.add(new Double(evaluate(child, QoSParameter)));
					}
					return (((QoSAttributeInterface) node.getQoSAttributesMap().get(
						QoSParameter)).choiceLogic(parameters));
				}
			}
		}
	}

	@SuppressWarnings("unchecked")
	public void inordre(TreeNode node) {
		list.add(node.getLabel());
		List childrenList = node.getChildren();
		if (childrenList != null) {
			System.out.println(node.getLabel() + " has " + childrenList.size()
					+ " children");
			for (Iterator it = childrenList.iterator(); it.hasNext();) {
				TreeNode child = (TreeNode) it.next();
				inordre(child);
			}
		}
	}

	public void setQoSAttribute(int index, Integer attributeType, double value) {
		actualIndex = 0;
		TreeNode node = getLeaf(this.rootNode, index);
		if (node != null) {
			node.setQoSAttribute(attributeType, value);
		}
	}

	public void setQoSAttributes(int index, HashMap listOfAttributes) {
		actualIndex = 0;
		TreeNode node = getLeaf(this.rootNode, index);
		if (node == null)
			DebugPrinter.print("index = " + index, 1);
		else
			node.setQoSAttributesMap(listOfAttributes);
	}

	public TreeNode getLeaf(TreeNode node, int index) {
		List childrenList = node.getChildren();
		if (childrenList == null) {
			if (actualIndex == index) {
				return (node);
			} else {
				actualIndex++;
			}
		} else {
			for (Iterator it = childrenList.iterator(); it.hasNext();) {
				TreeNode child = (TreeNode) it.next();
				TreeNode n = getLeaf(child, index);
				if (n != null) {
					return (n);
				}

			}
		}
		return null;
	}

	// --------------------------------------------------------------------------
	public void reinit() {
		TreeNode node = new TreeNode();
		TreeNode node2 = NodesFactory.createDefaultNode(this.QoSParameterMap);
		int i = 0;
		while (i < this.getNumberOfLeafs()) {
			actualIndex = 0;
			node = this.getLeaf(this.rootNode, i);
			node.setQoSAttributesMap(node2.getQoSAttributesMap());
			i++;
		}
	}

	// ---------------------Getters and
	// Setters----------------------------------
	public void setAutomaton(Automaton automaton) {
		this.automaton = automaton;
	}

	public void setRootNode(TreeNode rootNode) {
		this.rootNode = rootNode;
	}

	public void setNumberOfLeafs(int NumberOfLeafs) {
		this.NumberOfLeafs = NumberOfLeafs;
	}

	public void setQoSParametersList(HashMap ParametersList) {
		this.QoSParameterMap = ParametersList;
	}

	public Automaton getAutomaton() {
		return automaton;
	}

	public TreeNode getRootNode() {
		return rootNode;
	}

	public int getNumberOfLeafs() {
		return NumberOfLeafs;
	}

	public HashMap getQoSParameterList() {
		return this.QoSParameterMap;
	}

}
