/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.domparser.specializedparsers;

import java.lang.reflect.Field;

import org.w3c.dom.Node;

import fr.inria.icocoa.common.DebugPrinter;
import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.MapContainer;
import fr.inria.icocoa.servicemodel.domparser.toolobjects.SharedLock;
import fr.inria.icocoa.servicemodel.flow.CapabilityFlow;

/**
 * This parser is specialised for parsing Conversation
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Aug 3, 2007 3:09:15 PM
 */
public class ConversationParserLink extends AmigoSParserLink {

	/**
	 * The URI of the ontology containing the concept "CompositeProcess"
	 */
	private static final String MYURI1 = "http://www.daml.org/services/owl-s/1.1/Process.owl#";

	/**
	 * The URI of the ontology containing the concept "objList"
	 */
	private static final String MYURI2 = "http://www.daml.org/services/owl-s/1.1/generic/ObjectList.owl#";

	/**
	 * The URI of the ontology containing the concept "ID"
	 */
	private static final String MYURI3 = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";

	/** the prefix corresponding to the URI of the ontology MUURI1 */
	private String prefix1 = "";

	/** the prefix corresponding to the URI of the ontology MUURI2 */
	private String prefix2 = "";

	/** the prefix corresponding to the URI of the ontology MUURI3 */
	private String prefix3 = "";

	/**
	 * the string corresponding to the beginning of a list
	 */
	private static final String OBJLIST_FIRST = "first";

	/**
	 * the string corresponding to the rest of a list
	 */
	private static final String OBJLIST_REST = "rest";

	/** the string corresponding of the localName of the attribute giving an ID */
	private static final String RDF_ID = "ID";

	/**
	 * the string corresponding of the localName of the attribute giving an
	 * reference to an output declared somewhere else
	 */
	private static final String RDF_RESOURCE = "resource";

	/**
	 * the string corresponding of the localName of the rootNode that this
	 * subparser is responsible
	 */
	private static final String PROCESS_COMPOSITEPROCESS = "CompositeProcess";

	/**
	 * the string corresponding to the beginning of the declaration of a Perform
	 */
	private static final String PROCESS_PERFORM = "Perform";

	/**
	 * the string corresponding to the beginning of the declaration the process
	 * that has to be performed
	 */
	private static final String PROCESS_PROCESS = "process";

	/**
	 * the string corresponding to the beginning of the declaration of data
	 * binding
	 */
	private static final String PROCESS_HASDATAFROM = "hasDataFrom";

	/**
	 * the string corresponding to the beginning of the declaration of an input
	 * data binding
	 */
	private static final String PROCESS_INPUTBINDING = "InputBinding";

	/**
	 * the string corresponding to the beginning of the declaration of an input
	 * data binding
	 */
	private static final String PROCESS_OUTPUTBINDING = "OutputBinding";

	/**
	 * the string corresponding to the beginning of the declaration of the
	 * target variable name of a Binding
	 */
	private static final String PROCESS_TOPARAM = "toParam";

	/**
	 * the string corresponding to the beginning of the declaration of the
	 * source variable name of a Binding
	 */
	private static final String PROCESS_VALUESOURCE = "valueSource";

	/**
	 * the string corresponding to the beginning of the real declaration of the
	 * source variable name of a Binding
	 */
	private static final String PROCESS_THEVAR = "theVar";

	/**
	 * the string corresponding to the beginning of the real declaration of the
	 * source process name of a Binding
	 */
	private static final String PROCESS_FROMPROCESS = "fromProcess";

	/**
	 * *
	 * 
	 * @param lock
	 *            the shared lock
	 */
	public ConversationParserLink(SharedLock lock) {
		super(lock);
		this.setMyKeyword(this.getCompleteTagName(ConversationParserLink.PROCESS_COMPOSITEPROCESS));
	}

	/**
	 * @param mp *
	 * @param lock
	 *            the shared lock
	 */
	public ConversationParserLink(MapContainer mp, SharedLock lock) {
		super(mp, null, lock);
		this.setMyKeyword(this.getCompleteTagName(ConversationParserLink.PROCESS_COMPOSITEPROCESS));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.domparser.AmigoSParserLink#clone()
	 */
	@Override
	public AmigoSParserLink clone() {
		ConversationParserLink clone = new ConversationParserLink(
				this.getContainer(), this.getLock());
		clone.prefix1 = prefix1;
		clone.prefix2 = prefix2;
		clone.prefix3 = prefix3;
		return clone;
	}

	@Override
	public String[] getURIOfBasesConcept() {
		return new String[] { "prefix1", ConversationParserLink.MYURI1,
				"prefix2", ConversationParserLink.MYURI2, "prefix3",
				ConversationParserLink.MYURI3 };
	}

	@Override
	public void setPrefixValue(String prefixName, String prefixValue) {
		try {
			Field fd = this.getClass().getDeclaredField(prefixName);
			fd.set(this, prefixValue);
		} catch (SecurityException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
		} catch (NoSuchFieldException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
		} catch (IllegalArgumentException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
		} catch (IllegalAccessException e) {
			DebugPrinter.print(e.getStackTrace(), e.getMessage(),
				DebugPrinter.WARN);
		}
		this.setMyKeyword(this.getCompleteTagName(PROCESS_COMPOSITEPROCESS));
	}

	/**
	 * Get the string corresponding to the tag to recognise
	 * 
	 * @param concept
	 *            the name of the concept to recognise as tag
	 * @return the complete tag to recognise, <i>empty string</i> if an unknown
	 *         concept is given as parameter
	 */
	private String getCompleteTagName(String concept) {
		if (concept.equals(PROCESS_COMPOSITEPROCESS))
			return this.prefix1 + ":" + concept;
		if (concept.equals(PROCESS_PERFORM))
			return this.prefix1 + ":" + concept;
		if (concept.equals(PROCESS_PROCESS))
			return this.prefix1 + ":" + concept;
		if (concept.equals(PROCESS_HASDATAFROM))
			return this.prefix1 + ":" + concept;
		if (concept.equals(PROCESS_INPUTBINDING))
			return this.prefix1 + ":" + concept;
		if (concept.equals(PROCESS_OUTPUTBINDING))
			return this.prefix1 + ":" + concept;
		if (concept.equals(PROCESS_TOPARAM))
			return this.prefix1 + ":" + concept;
		if (concept.equals(PROCESS_THEVAR))
			return this.prefix1 + ":" + concept;
		if (concept.equals(PROCESS_VALUESOURCE))
			return this.prefix1 + ":" + concept;
		if (concept.equals(PROCESS_FROMPROCESS))
			return this.prefix1 + ":" + concept;
		if (concept.equals(OBJLIST_FIRST))
			return this.prefix2 + ":" + concept;
		if (concept.equals(OBJLIST_REST))
			return this.prefix2 + ":" + concept;
		if (concept.equals(RDF_ID))
			return this.prefix3 + ":" + concept;
		if (concept.equals(RDF_RESOURCE))
			return this.prefix3 + ":" + concept;
		return "";
	}

	@Override
	protected Object parseNextObject_MethodContent(Node rootNode)
			throws ParsingException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	protected void parse_MethodContent(Node rootNode) throws ParsingException {
		Object out_obj = this.parseNextObject(rootNode);

		if (out_obj != null) {
			if (out_obj instanceof CapabilityFlow) {
				CapabilityFlow conversation = (CapabilityFlow) out_obj;
				DebugPrinter.print(
					"ConversationParserLink::parse(..) ->Output = "
							+ conversation, DebugPrinter.INFO);

				// TODO register it somewhere
			} else {
				DebugPrinter.print("The node starting with " + rootNode
						+ " has not be recognised as a Conversation !",
					DebugPrinter.WARN);
			}
		}
	}

	@Override
	protected String myToString() {
		return super.displayField();
	}

}
