/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.domparser.toolobjects;

/**
 * This class contains a lock that will be shared by all instance of ParserLink.
 * This lock will be used to tag it as ignorable in case of a tag associated to
 * a Parser has not be defined in the file.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Sep 3, 2007 5:01:34 PM
 */
public class SharedLock implements Cloneable {

	/** The synchronised lock */
	private boolean locked = false;

	/**
	 * @return the locked
	 */
	public synchronized boolean isLocked() {
		return locked;
	}

	/**
	 * @param locked
	 *            the locked to set
	 */
	public synchronized void setLocked(boolean locked) {
		this.locked = locked;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#clone()
	 */
	@Override
	public SharedLock clone() throws CloneNotSupportedException {
		SharedLock clone = new SharedLock();
		clone.setLocked(this.isLocked());
		return clone;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "SharedLock[" + this.isLocked() + "]";
	}

}
