/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.flow;

import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import fr.inria.icocoa.servicemodel.ControlConstruct;
import fr.inria.icocoa.servicemodel.ParameterBinding;

/**
 * This action is a composite part of flow that will describe a list of action
 * that have to be done in the given order.
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Aug 31, 2007 1:43:49 PM
 */
public class SequenceAction extends CompositeCapabilityFlow {

	/**
	 * Serial version number
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Create a new CompositeCapabilityFlow dedicated to the Sequence
	 */
	public SequenceAction() {
		super(ControlConstruct.SEQUENCE);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#toString(java.lang.String)
	 */
	@Override
	public String toString(String prefix) {
		/*
		 * String result = prefix + "- ["; // inputs bindings for (Entry<String,
		 * ParameterBinding> input : this.getInputsBinding() .entrySet()) {
		 * result += input.getValue().getSourceVariable() + "->" +
		 * input.getValue().getTargetVariable() + ","; }
		 * 
		 * result += "]\n"+prefix+" SEQUENCE\n"+prefix+" ["; // outputs bindings
		 * for (Entry<String, ParameterBinding> output :
		 * this.getOutputsBinding() .entrySet()) { result +=
		 * output.getValue().getSourceVariable() + "->" +
		 * output.getValue().getTargetVariable() + ","; }
		 * 
		 * result += "]\n";
		 */

		String result = prefix + "- SEQUENCE\n";

		// display children
		for (CapabilityFlow child : this.getChildren())
			if (child != null) {
				result += child.toString(prefix + "\t");
			}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#clone()
	 */
	@Override
	public SequenceAction clone() throws CloneNotSupportedException {
		SequenceAction clone = new SequenceAction();

		// clone child
		for (CapabilityFlow child : this.getChildren()) {
			CapabilityFlow childclone = child.clone();
			// update FuturCapabilityFlow reference of composite child
			if (childclone instanceof FuturCapabilityFlow)
				((FuturCapabilityFlow) childclone).addNewReferenceOnI(clone);

			clone.addChild(childclone);
		}

		// clone InputsBindings
		for (Entry<String, ParameterBinding> entry : this.getInputsBinding().entrySet())
			clone.getInputsBinding().put(entry.getKey(),
				entry.getValue().clone());

		// clone OutputsBindings
		for (Entry<String, ParameterBinding> entry : this.getOutputsBinding().entrySet())
			clone.getOutputsBinding().put(entry.getKey(),
				entry.getValue().clone());

		return clone;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see fr.inria.icocoa.servicemodel.flow.CapabilityFlow#generateBPELFragment(java.lang.String,
	 *      java.util.Map, java.util.Map, java.util.Map)
	 */
	@Override
	public String generateBPELFragment(String prefix,
			Map<String, String> service_name,
			Map<String, String> service_portType,
			Map<String, String> service_nameSpace) {
		String bpel_str = prefix + "<sequence>\n";

		// generate bpel script by recursion
		for (Iterator<CapabilityFlow> it = this.getChildren().iterator(); it.hasNext();)
			bpel_str += it.next().generateBPELFragment(prefix, service_name,
				service_portType, service_nameSpace);

		bpel_str += prefix + "</sequence>\n\n";
		return bpel_str;
	}

}
