package fr.inria.icocoa.servicemodel.parsers;

import java.io.IOException;
import java.util.List;

import fr.inria.icocoa.exception.ParsingException;
import fr.inria.icocoa.servicemodel.Service;

/**
 * 
 * This interface describe the minimum set of fonction required for a Service Parser
 *
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Aug 21, 2008 5:36:27 PM
 */
public abstract class ServiceParser {

	/**
	 * The parser that is currently used
	 */
	protected static ServiceParser currentParser = null;

	/**
	 * This method create the Service from a file
	 * 
	 * @param uri
	 *            the uri of this file
	 * @param getGrounding
	 *            put the value to "true" if you want need to get the grounding,
	 *            "false" otherwise
	 * @return a complete Service Object
	 * @throws ParsingException
	 *             if an error occurs (see Exception message for more
	 *             information)
	 */
	public abstract Service parseProcess(String uri, boolean getGrounding)
			throws ParsingException;

	/**
	 * This method create the Service from a file
	 * 
	 * @param Owl_sContent
	 *            the description file content (Amigo-S language)
	 * @param getGrounding
	 *            put the value to "true" if you want need to get the grounding,
	 *            "false" otherwise
	 * @return a complete Service Object
	 * @throws ParsingException
	 *             if an error occurs (see Exception message for more
	 *             information)
	 * @throws IOException
	 *             if an error occurs with a file
	 */
	public abstract Service parseProcessFromString(String Owl_sContent,
			boolean getGrounding) throws IOException, ParsingException;

	/**
	 * Parse an Amigo-S description to generate a equivalent Service object. The
	 * wsdl URI given as second parameter will be associated to the description.
	 * The grounding part is also parsed.
	 * 
	 * @param semanticDescriptionURI
	 *            the URI of the file containing the amigo-S description
	 * 
	 * @param wsdlURI
	 *            the URi of the file containg the WSDL description
	 * @return a Service object equivalent to the Amigo-S description
	 * @throws ParsingException
	 *             if error occurs during parsing
	 */
	public abstract Service parseProcess(String semanticDescriptionURI,
			String wsdlURI) throws ParsingException;

	/**
	 * Get a hashcode corresponding to the list of imports of the last service
	 * description read
	 * 
	 * @return a String if a file has been already read, <b>null</b> otherwise
	 */
	public abstract String getImportsHashCode();
	
	/**
	 * Get the list of imports of the last service
	 * description read
	 * 
	 * @return the list of the URI of the ontologies imported if a file has been already read, <b>null</b> otherwise
	 */
	public abstract List<String> getListOfImports();

	/**
	 * @return the currentParser
	 */
	public static ServiceParser getCurrentParser() {
		return currentParser;
	}
}