/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.semantictype;

/**
 * Represent a Code for an Encoded Semantic Type. 
 *
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Apr 8, 2008 10:00:04 AM
 */
public interface Code {
	
	/**
	 * Check if the current code correspond to a concept subsum of the concept given by <i>other</i> code
	 * @param other the code to compare with
	 * @return <b>true</b> if current code is subsum of <i>other</i>, <b>false</b> otherwise 
	 */
	public boolean isSubsum(Code other);
	
	/**
	 * Check if the current code correspond to a concept plugin of the concept given by <i>other</i> code
	 * @param other the code to compare with
	 * @return <b>true</b> if current code is plugin of <i>other</i>, <b>false</b> otherwise 
	 */
	public boolean isPlugin(Code other);
	
	/**
	 * Check if both code correspond to equivalent concept 
	 * @param other the Code of the other concept 
	 * @return <b>true</b> if code correspond to equivalent concept, <b>false</b> otherwise 
	 */
	public boolean isExact(Code other);

}
