/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.semantictype;

import java.util.ArrayList;
import java.util.List;

import fr.inria.icocoa.servicemodel.semantictype.encryptionmanagement.EncodedOntologiesManager;
import fr.inria.icocoa.servicemodel.semantictype.encryptionmanagement.Encoder;

/**
 * Extension of the SemanticType to manage encryption of this type
 *
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Mar 5, 2008 3:51:08 PM
 */
public abstract class EncodedSemanticType extends StringSemanticType {
	

	/**
	 * The manager of encoded ontologies
	 */
	protected static EncodedOntologiesManager ontManager = null;
	
	/**
	 * The associated Encoder class
	 */
	protected static Class<? extends Encoder> associatedEncoder = null;
	
	/**
	 * The associated Code class
	 */
	protected static Class<? extends Code> associatedCodeClass = null;
	
	/**
	 * The list of code corresponding to the object
	 */
	protected List<Code> listOfCodes = new ArrayList<Code>();
		
	/**
	 * This part of code is done when this class is load for the first time<br>
	 * Load the EncodedOntologiesManager
	 */
	static{
		EncodedSemanticType.ontManager = EncodedOntologiesManager.getInstance();
	}
	
	/**
	 * 
	 */
	public EncodedSemanticType() {
		super();
	}

	/**
	 * @param typeName
	 * @param ontology
	 */
	public EncodedSemanticType(String typeName, String ontology) {
		super(typeName, ontology);
	}

	/* (non-Javadoc)
	 * @see fr.inria.icocoa.servicemodel.semantictype.StringSemanticType#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		super.finalize();
	}

	/* (non-Javadoc)
	 * @see fr.inria.icocoa.servicemodel.semantictype.StringSemanticType#setType(java.lang.String)
	 */
	@Override
	public void setType(String newType) {
		super.setType(newType);
	}
	
	/**
	 * Check if the other concept is equals or equivalent of the other concept
	 * @param other the encoded value of another concept
	 * @return <i>true</i> if both concept are equals or equivalent, <i>false</i> otherwise.
	 */
	public abstract boolean isEquals(EncodedSemanticType other);
	
	/**
	 * Check if the other concept is subsum of the other concept
	 * @param other the encoded value of another concept
	 * @return <i>true</i> if the other concept is subsum of the current object
	 */
	public abstract boolean isSubsum(EncodedSemanticType other);
	
	/**
	 * Check if the other concept is plugin of the other concept
	 * @param other the encoded value of another concept
	 * @return <i>true</i> if the other concept is Plugin of the current object
	 */
	public abstract boolean isPlugin(EncodedSemanticType other);

	

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return super.toString();
	}

	/**
	 * @return the associatedCodeClass
	 */
	public Class<? extends Code> getAssociatedCodeClass() {
		return associatedCodeClass;
	}

	/**
	 * @return the associatedEncoder
	 */
	public Class<? extends Encoder> getAssociatedEncoder() {
		return associatedEncoder;
	}

	/**
	 * @return the listOfCodes
	 */
	public List<Code> getListOfCodes() {
		return listOfCodes;
	}

	/**
	 * @param listOfCodes the listOfCodes to set
	 */
	public void setListOfCodes(List<Code> listOfCodes) {
		this.listOfCodes = listOfCodes;
	}

	
	
	
	

}
