/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.semantictype;

import java.io.Serializable;

/**
 * This class represent the Semantic concept of an object.<br><br>
 * All subclasses required to have at least two constructors :
 * 	- one without parameters
 *  - one using two String (type name and ontology URL)
 *
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Mar 4, 2008 11:27:18 AM
 */
public abstract class SemanticType implements Serializable{
	
	protected final static String UNKNOWN = "Unknown";	
	
	/**
	 * Get the URL of the ontology
	 * @return the ontology URL
	 */
	public abstract String getOntology();
	
	/**
	 * Get the name of the semantic concept used
	 * @return the name of the semantic Type
	 */
	public abstract String getTypeName();
	
	/**
	 * Set a new URL to the Ontology
	 * @param ontology the ontology to set
	 */
	public abstract void setOntology(String ontology);
	
	/**
	 * Set a new Value to name of the concept
	 * @param typeName the typeName to set
	 */
	public abstract void setTypeName(String typeName) ;
	
	/**
	 * Set a new rdtType value
	 * 
	 * @param newType :
	 *            the new type to set
	 */
	public abstract void setType(String newType);
	
	/**
	 * Get the rdtType value
	 * @return the string representation of the type (ontology#typeName)
	 */
	public String getType(){
		return this.getOntology()+"#"+this.getTypeName();
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public abstract boolean equals(Object obj);
	
	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public abstract int hashCode();
	
	 
	

}
