/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.semantictype;

import fr.inria.icocoa.common.Tools;

/**
 *  Implementation of SemanticType that doesn't use any encryption, but just store type as string
 
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Mar 4, 2008 11:30:24 AM
 */
public class StringSemanticType extends SemanticType {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The typeName (ontology class) of the input (without the ontology
	 * extension)
	 */
	protected String typeName = UNKNOWN;

	/**
	 * The URL of the ontology used
	 */
	protected String ontology = UNKNOWN;

	/**
	 * Default constructor
	 * 
	 */
	public StringSemanticType() {
	}

	/**
	 * @param typeName
	 * @param ontology
	 */
	public StringSemanticType(String typeName, String ontology) {
		if (typeName != null)
			this.typeName = typeName;
		if (ontology != null)
			this.ontology = ontology;
	}

	/**
	 * @return the ontology
	 */
	public String getOntology() {
		return ontology;
	}

	/**
	 * @param ontology
	 *            the ontology to set
	 */
	public void setOntology(String ontology) {
		this.ontology = ontology;
	}

	/**
	 * @return the typeName
	 */
	public String getTypeName() {
		return typeName;
	}

	/**
	 * @param typeName
	 *            the typeName to set
	 */
	public void setTypeName(String typeName) {
		this.typeName = typeName;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable {
		this.typeName = null;
		this.ontology = null;
	}

	/**
	 * Set a new rdtType value
	 * 
	 * @param newType :
	 *            the new type to set
	 */
	public void setType(String newType) {
		if (newType != null) {
			String _typeName = Tools.getSecondPart(newType);
			String _ontology = Tools.getFirstPart(newType);

			if (_typeName != null)
				this.typeName = _typeName;
			if (_ontology != null)
				this.ontology = _ontology;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME
				* result
				+ ((this.getOntology() == null) ? 0 : this.getOntology()
						.hashCode());
		result = PRIME
				* result
				+ ((this.getTypeName() == null) ? 0 : this.getTypeName()
						.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final StringSemanticType other = (StringSemanticType) obj;
		if (this.getOntology() == null) {
			if (other.getOntology() != null)
				return false;
		} else if (!this.getOntology().equals(other.getOntology()))
			return false;
		if (this.getTypeName() == null) {
			if (other.getTypeName() != null)
				return false;
		} else if (!this.getTypeName().equals(other.getTypeName()))
			return false;
		return true;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return this.getOntology()+"#"+this.getTypeName();
	}

}
