/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.semantictype.encryptionmanagement;

import java.io.Serializable;
import java.math.BigInteger;

import fr.inria.icocoa.servicemodel.semantictype.Code;

/**
 * a wrapper to convert BigInteger into Code instance
 *
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Apr 8, 2008 3:38:46 PM
 */
public class BigIntegerCode implements Code, Serializable{
	
	/**
	 * serial ID
	 */
	private static final long serialVersionUID = 1L;
	
	/**
	 * The value carriered by this object 
	 */
	private BigInteger value = null;

	/**
	 * @param value
	 */
	public BigIntegerCode(BigInteger value) {
		this.value = value;
	}
	
	/**
	 * @return the value
	 */
	public BigInteger getValue() {
		return value;
	}

	/**
	 * @param value the value to set
	 */
	public void setValue(BigInteger value) {
		this.value = value;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + ((value == null) ? 0 : value.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final BigIntegerCode other = (BigIntegerCode) obj;
		if (value == null) {
			if (other.value != null)
				return false;
		} else if (!value.equals(other.value))
			return false;
		return true;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return value.toString();
	}

	/* (non-Javadoc)
	 * @see fr.inria.icocoa.servicemodel.semantictype.Code#isExact(fr.inria.icocoa.servicemodel.semantictype.Code)
	 */
	public boolean isExact(Code other) {
		BigInteger otherValue = ((BigIntegerCode)other).getValue(); 
		return this.getValue().equals(otherValue);
	}

	/* (non-Javadoc)
	 * @see fr.inria.icocoa.servicemodel.semantictype.Code#isPlugin(fr.inria.icocoa.servicemodel.semantictype.Code)
	 */
	public boolean isPlugin(Code other) {
		BigInteger otherValue = ((BigIntegerCode)other).getValue(); 
		return otherValue.remainder(this.getValue()).intValue()==0;
	}

	/* (non-Javadoc)
	 * @see fr.inria.icocoa.servicemodel.semantictype.Code#isSubsum(fr.inria.icocoa.servicemodel.semantictype.Code)
	 */
	public boolean isSubsum(Code other) {
		return other.isPlugin(this);
	}

	
	
	
	

}
