/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel.toolsobjects;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import fr.inria.icocoa.common.Tools;
import fr.inria.icocoa.servicemodel.ParameterBinding;

/**
 * CapabilityMap.java
 * 
 * @author <a href="mailto:Galip-Ferda.Tartanoglu@inria.fr">Ferda Tartanoglu</a> -
 *         <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 */
public class CapabilityMap {

	/**
	 * The URI of the capability
	 */
	private String capability = "";

	/**
	 * THe URI of the process
	 */
	private String process = "";

	/**
	 * The Rdf type of the capability
	 */
	private String type = "";

	// data bindings
	/** input binding FROM=TO */
	public Map<String, ParameterBinding> inputBinding = new HashMap<String, ParameterBinding>();

	/** output binding TO=FROM */
	public Map<String, ParameterBinding> outputBinding = new HashMap<String, ParameterBinding>();

	/**
	 * @param capability
	 * @param process
	 * @param type
	 */
	public CapabilityMap(String capability, String process, String type) {
		this.capability = capability;
		this.process = process;
		this.type = type;
	}

	@Override
	public String toString() {
		String result = "CapabilityMap[\nType=\"" + this.getType()
				+ "\",\nCapability=\"" + this.getCapability()
				+ "\",\nProcess=\"" + this.getProcess()
				+ "\",\nInputBindings=\"";
		for (Entry<String, ParameterBinding> entry : this.getInputBinding().entrySet()) {
			result += "\n\t-" + entry.getKey() + " = "
					+ entry.getValue().toString();
		}
		result += "\",\nOutputBindings=\"";
		for (Entry<String, ParameterBinding> entry : this.getOutputBinding().entrySet()) {
			result += "\n\t-" + entry.getKey() + " = "
					+ entry.getValue().toString();
		}
		result += "\"]";
		return result;
	}

	/**
	 * @return the capability
	 */
	public String getCapability() {
		return capability;
	}

	/**
	 * @param capability
	 *            the capability to set
	 */
	public void setCapability(String capability) {
		this.capability = capability;
	}

	/**
	 * @return the process
	 */
	public String getProcess() {
		return process;
	}

	/**
	 * @param process
	 *            the process to set
	 */
	public void setProcess(String process) {
		this.process = process;
	}

	/**
	 * @return the type
	 */
	public String getType() {
		return type;
	}

	/**
	 * @param type
	 *            the type to set
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * @return the inputBinding
	 */
	public Map<String, ParameterBinding> getInputBinding() {
		return inputBinding;
	}

	/**
	 * @param inputBinding
	 *            the inputBinding to set
	 */
	public void setInputBinding(Map<String, ParameterBinding> inputBinding) {
		this.inputBinding = inputBinding;
	}

	/**
	 * @return the outputBinding
	 */
	public Map<String, ParameterBinding> getOutputBinding() {
		return outputBinding;
	}

	/**
	 * @param outputBinding
	 *            the outputBinding to set
	 */
	public void setOutputBinding(Map<String, ParameterBinding> outputBinding) {
		this.outputBinding = outputBinding;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((capability == null) ? 0 : capability.hashCode());
		result = prime * result
				+ ((inputBinding == null) ? 0 : inputBinding.hashCode());
		result = prime * result
				+ ((outputBinding == null) ? 0 : outputBinding.hashCode());
		result = prime * result + ((process == null) ? 0 : process.hashCode());
		result = prime * result + ((type == null) ? 0 : type.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final CapabilityMap other = (CapabilityMap) obj;
		if (capability == null) {
			if (other.capability != null)
				return false;
		} else if (!capability.equals(other.capability))
			return false;
		if (inputBinding == null) {
			if (other.inputBinding != null)
				return false;
		} else if (!Tools.compareMap(this.inputBinding, other.inputBinding))
			return false;
		if (outputBinding == null) {
			if (other.outputBinding != null)
				return false;
		} else if (!Tools.compareMap(this.outputBinding, other.outputBinding))
			return false;
		if (process == null) {
			if (other.process != null)
				return false;
		} else if (!process.equals(other.process))
			return false;
		if (type == null) {
			if (other.type != null)
				return false;
		} else if (!type.equals(other.type))
			return false;
		return true;
	}

}
