/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.toolsobjects;

/**
 * This class is a container for two objects of undetermined type
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * Creation Date : Oct 22, 2007 3:00:35 PM
 * 
 * @param <K>
 *            the java type of the first object
 * @param <T>
 *            the java type of the second object
 * 
 */
public class Couple<K, T> {

	/** the first object */
	private K first = null;

	/** the second object */
	private T second = null;

	/**
	 * @param first
	 * @param second
	 */
	public Couple(K first, T second) {
		this.first = first;
		this.second = second;
	}

	/**
	 * @return the first
	 */
	public K getFirst() {
		return first;
	}

	/**
	 * @param first
	 *            the first to set
	 */
	public void setFirst(K first) {
		this.first = first;
	}

	/**
	 * @return the second
	 */
	public T getSecond() {
		return second;
	}

	/**
	 * @param second
	 *            the second to set
	 */
	public void setSecond(T second) {
		this.second = second;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + ((first == null) ? 0 : first.hashCode());
		result = PRIME * result + ((second == null) ? 0 : second.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final Couple<K, T> other = (Couple<K, T>) obj;
		if (first == null) {
			if (other.first != null)
				return false;
		} else if (!first.equals(other.first))
			return false;
		if (second == null) {
			if (other.second != null)
				return false;
		} else if (!second.equals(other.second))
			return false;
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return super.toString() + "=\nFirst:\n------\n"
				+ this.getFirst().toString() + "\nSecond:\n-------\n"
				+ this.getSecond().toString();
	}

}
