/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */
package fr.inria.icocoa.servicemodel.toolsobjects;

import java.io.Serializable;

/**
 * This object could be share by two thread to indicate "a place" where storing
 * a result whitout without waiting the end of the execution. The Futur will
 * updated by the "calculator" thread before its terminaison. When this thread
 * shut down, the second one could access the result ! (It could access before,
 * but the result could not to be updated).<br>
 * 
 * <br>
 * The result is initialized to <b>null</b>.
 * 
 * 
 * @author <a href="mailto:sebastien.bianco@inria.fr">sebastien bianco</a> - <a
 *         href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 * 
 * @param K
 *            the type of object that will be stored
 * 
 * Creation Date : Aug 8, 2007 10:20:46 AM
 */
public class Futur<K> implements Cloneable, Serializable {

	/**
	 * serial version ID
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * The object that will be updated by the calculator thread before its
	 * terminaison
	 */
	private K value = null;

	/**
	 * Construct future object by giving the value to store
	 * 
	 * @param value
	 *            the value to store
	 */
	public Futur(K value) {
		this.value = value;
	}

	/**
	 * Default constructor, initialise value to null;
	 * 
	 */
	public Futur() {
	}

	/**
	 * @return the value
	 */
	public synchronized K getValue() {
		return value;
	}

	/**
	 * @param value
	 *            the value to set
	 */
	public synchronized void setValue(K value) {
		this.value = value;
	}

	/**
	 * This method generate a String to represent a Capability object
	 * 
	 * @param prefix
	 *            the prefix add to each new line
	 * @return the generate String
	 */
	public String toString(String prefix) {
		if (this.getValue() == null)
			return prefix + "Futur<null>";
		else
			return prefix + this.getValue().toString();
	}

	@Override
	public String toString() {
		return this.toString("");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#clone()
	 */
	@Override
	public Object clone() throws CloneNotSupportedException {
		return new Futur<K>(this.getValue());
	}

}
