/**
 ##############################################################################
 # Copyright (C) 2006-2010 INRIA
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 2 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ##############################################################################
 * 
 */

package fr.inria.icocoa.servicemodel.toolsobjects;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * ProcessElements.java<br>
 * This class store the description of a process
 * 
 * @author <a href="mailto:Galip-Ferda.Tartanoglu@inria.fr">Ferda Tartanoglu</a> -
 *         <a href="http://www-rocq.inria.fr/arles">Arles Project - INRIA</a>
 */
public class ProcessElements {

	/** The Streing coding the ControlConstruct */
	private String control = "";

	/** The list of element composing the process */
	private ArrayList<CapabilityMap> element = new ArrayList<CapabilityMap>();

	@Override
	public String toString() {
		String result = "control = " + getControl() + "\nCapabilityMap :\n";
		for (int i = 0; i < element.size(); i++) {
			result += "\t" + element.get(i) + "\n";
		}
		return result;
	}

	/**
	 * @param control
	 *            the control to set
	 */
	public void setControl(String control) {
		this.control = control;
	}

	/**
	 * @return the control
	 */
	public String getControl() {
		return control;
	}

	/**
	 * this method permit to add an element in the list of elements
	 * 
	 * @param cm
	 *            the element we want to add
	 */
	public void addElement(CapabilityMap cm) {
		this.element.add(cm);
	}

	/**
	 * this method permit to remove an element from the list of elements
	 * 
	 * @param cm
	 *            the element we want to remove
	 */
	public void removeElement(CapabilityMap cm) {
		this.element.remove(cm);
	}

	/**
	 * This method permit to get the element store at the given position in the
	 * list
	 * 
	 * @param i :
	 *            the position of the element we want to get
	 * @return the asked Element
	 */
	public CapabilityMap getElement(int i) {
		return this.element.get(i);
	}

	/**
	 * Get the number of elements composing the process
	 * 
	 * @return the size of the list of elements
	 */
	public int size() {
		return this.element.size();
	}

	/**
	 * Get an iterator on the list of elements
	 * 
	 * @return the iterator of list "element"
	 */
	public Iterator<CapabilityMap> getIterator() {
		return this.element.iterator();
	}

}
